function PolePlaceI(lambda)
%
% Places the poles for Integral feedback control
% i.e. when,
%           uu = KK * Integration[ss * dt]
%
% Can be used when C2 is not zero.
%
% --------------------------------------------------------------------
% Given the position of the poles, 'lamb', in the cplx plane, determines the gain matrix K
% K = [k1 k2 k3 ... kn]
%
% The equation to be solved for K is,
%
%         --                                                             --
%        ||                                |                              ||
%        || inv(A-C2*K)*(B+C1*K) - lamb*I  |     inv(A-C2*K)*C0*K         ||
%        ||                                |                              ||
%    det ||--------------------------------|------------------------------||  =  0
%        ||                                |                              ||
%        ||             I                  |           -lamb*I            ||
%        ||                                |                              ||
%         --                                                             --
%
% There are 2n poles (i.e. 2n 'lamb's), which gives 2n equations.
% But if we assume that A, B, C0, C1, C2 and K have only real elements,
% 'lamb' will consist of complex conjugates.
% Hence, lamb has n elements.
% Solve for K using Newton Raphson method of iteration
% where n = stateVarNo

% State space formulation
global stateVarNo           % No of state variables
global ctrlParmNo           % No of control parameters
global ExpressionVectorSize % No of elements in an 'expression vector'
global ss                   % State vector
global AA BB CC0 CC1 CC2 DD % System equation matrices
global KK                   % Gain Matrix
global LastEqnNo            % Variable to keep track of the number of equation incorporated
global lamb KKord
global last_del_g

lamb = lambda;
last_del_g = ones(stateVarNo, stateVarNo);

LambOrd=100;
%thePoles = (-(0.7+0.3*rand(stateVarNo, 1))*2 + (2*rand(stateVarNo, 1))*i)*ord;
%KKord = max(max(abs(BB)))/max(abs(CC0)),
%KKord = min([max(max(abs(BB)))/max(abs(CC0)) max(max(abs(BB)))/max(abs(CC1)) max(max(abs(AA)))/max(abs(CC2))]);
KKord = 1;
%tmp_KK = (ones(1, stateVarNo) - rand(1, stateVarNo)/2) * KKord;  % intial guess of K
tmp_KK = zeros(1, stateVarNo);

error_chk = .00001;

the_last_error = inf;
maxIter=1000;
for a=1:maxIter,
    %a,
    if isThisKKokay(tmp_KK)==1
        break;
    end
    last_tmp_KK = tmp_KK;
    
    %g(tmp_KK),
    %Mat_del_g,
    Mat_del_g = del_g(tmp_KK);
    %Mat_del_g,
    %Mat_del_g(:,1),
    %Mat_del_g(:,1)==zeros(stateVarNo,1),
    %pause;
    
    % choosing the g's which are not independent of K.
    gIndexCount = 0;
    for b=1:stateVarNo,
        if not(dot(Mat_del_g(:,b),Mat_del_g(:,b))==0)
            gIndexCount = gIndexCount+1;
            gIndex(gIndexCount) = b;
        end
    end
    tmp_Mat_del_g = Mat_del_g(:,gIndex);
    
    kIndexCount = 0;
    for b=1:stateVarNo,
        if not(dot(tmp_Mat_del_g(b,:),tmp_Mat_del_g(b,:))==0)
            kIndexCount = kIndexCount+1;
            kIndex(kIndexCount) = b;
        end
    end

    
    %Mat_del_g(:,1),
    %a,
    %gIndexCount,
    % Randomly choosing choosing rows/columns to make a square 'Mat_del_g' -- choosing with equal probability
    maxIndexCount = max(gIndexCount,kIndexCount);
    minIndexCount = min(gIndexCount,kIndexCount);
    chooseFlag = zeros(1,maxIndexCount);
    for b=1:minIndexCount,
        nextRand = 1+round(rand*(maxIndexCount-b));
        count1 = 0;
        for c=1:maxIndexCount,
            if chooseFlag(c)==0
                count1=count1+1;
            end
            if count1==nextRand
                chooseFlag(c)=1;
                break;
            end
        end
    end
    if kIndexCount>gIndexCount  % To eliminate a few k's
        kIndexCount=0;
        for b=1:maxIndexCount,
            if chooseFlag(b)==1
                kIndexCount=kIndexCount+1;
                kIndexIndex(kIndexCount) = b;
            end
        end
        kIndex = kIndex(kIndexIndex);
    elseif kIndexCount<gIndexCount  % To eliminate a few g's
        gIndexCount=0;
        for b=1:maxIndexCount,
            if chooseFlag(b)==1
                gIndexCount=gIndexCount+1;
                gIndexIndex(gIndexCount) = b;
            end
        end
        gIndex = gIndex(gIndexIndex);
    end
    %kIndex,
    %gIndex
    
    sub_g = g(tmp_KK);
    sub_g = sub_g(gIndex);
    sub_Mat_del_g = Mat_del_g(:,gIndex);
    sub_Mat_del_g = sub_Mat_del_g(kIndex,:);
    tmp_KK(kIndex) = tmp_KK(kIndex) - real(sub_g*inv(sub_Mat_del_g));
    
    if (dot(tmp_KK, tmp_KK) == 0) && (dot(last_tmp_KK, last_tmp_KK) == 0),
        break;
    end
    
    the_error = dot((tmp_KK-last_tmp_KK), (tmp_KK-last_tmp_KK))/dot((tmp_KK+last_tmp_KK), (tmp_KK+last_tmp_KK));
    %if the_error < error_chk
        %break;
        %end
    
    if mod(a+1,10)==0
        sprintf('Iteration no.: %d -- Error = %d', a, the_error)
    end
    if isnan(the_error) || isinf(the_error)  % divergence
        sprintf('WARNING!! Divergence in iteration no.: %d -- Re-initiating iteration!!', a)
        %tmp_ddq_p = ddq_p + max(abs(ddq_p))*(ones(1, modesNo) - rand(1, modesNo)/2);
        %tmp_KK = (ones(1, stateVarNo) - rand(1, stateVarNo)/2)*KKord;
        tmp_KK = zeros(1, stateVarNo);
        lamb = (-(0.7+0.3*rand(stateVarNo, 1))*2 + (10*rand(stateVarNo, 1))*i)*LambOrd;
        last_del_g = ones(stateVarNo, stateVarNo);
        %a = 0;
    end
    the_last_error = the_error;
end

if not(a==maxIter)
    sprintf('Convergence in %d iterations', a)
    
    tmpMat = inv(AA-CC2*tmp_KK);
    MM11 = tmpMat*(BB+CC1*tmp_KK);
    MM12 = tmpMat*CC0*tmp_KK;
    MM21 = eye(stateVarNo);
    MM22 = zeros(stateVarNo);
    MM = [ MM11 MM12 ; MM21 MM22 ];    % / (0.5e100/stateVarNo);
    %MMord = max(max(abs(MM)));
    %MM = MM/lamb(n);
    %rank(MM),
    %eig(MM),
    %lamb,
    
end

KK = tmp_KK;


% ----------------------------
%       End of Function
% ============================




function [g]=g(this_KK)

%
% given a particular K and lamb, this function calculates the vector g
%
%
%               --                                                                --
%              ||                                   |                              ||
%              || inv(A-C2*K)*(B+C1*K) - lamb(i)*I  |     inv(A-C2*K)*C0*K         ||
%              ||                                   |                              ||
% g(i) =   det ||-----------------------------------|------------------------------|| /  1e100
%              ||                                   |                              ||
%              ||             I                     |         -lamb(i)*I           ||
%              ||                                   |                              ||
%               --                                                                --
%
% i = 1 to stateVarNo
%

% State space formulation
global stateVarNo           % No of state variables
global ctrlParmNo           % No of control parameters
global ExpressionVectorSize % No of elements in an 'expression vector'
global ss                   % State vector
global AA BB CC0 CC1 CC2 DD % System equation matrices
global KK                   % Gain Matrix
global LastEqnNo            % Variable to keep track of the number of equation incorporated
global lamb


for n = 1:stateVarNo,
    tmpMat = inv(AA-CC2*this_KK);
    MM11 = tmpMat*(BB+CC1*this_KK) - lamb(n)*eye(stateVarNo);
    MM12 = tmpMat*CC0*this_KK;
    MM21 = eye(stateVarNo);
    MM22 = - lamb(n)*eye(stateVarNo);
    MM = [ MM11 MM12 ; MM21 MM22 ];    % / (0.5e100/stateVarNo);
    %MMord = max(max(abs(MM)));
    MM = MM/lamb(n);
    %rank(MM),
    tmpg(n) = det(MM);
end

g = tmpg;

% ----------------------------
%       End of Function
% ============================



function [del_g]=del_g(this_KK)

% returns the matrix
%
% del * g' =
%           d(g(1))/d(KK(1))           d(g(2))/d(KK(1))           ...      d(g(stateVarNo))/d(KK(1))
%           d(g(1))/d(KK(2))           d(g(2))/d(KK(2))           ...      d(g(stateVarNo))/d(KK(2))
%                  :                           :                                       :
%                  :                           :         d(g(b))/d(KK(a))              :
%                  :                           :                                       :
%           d(g(1))/d(KK(stateVarNo))  d(g(2))/d(KK(stateVarNo))   ...      d(g(stateVarNo))/d(KK(stateVarNo))

% State space formulation
global stateVarNo           % No of state variables
global ctrlParmNo           % No of control parameters
global ExpressionVectorSize % No of elements in an 'expression vector'
global ss                   % State vector
global AA BB CC0 CC1 CC2 DD % System equation matrices
global KK                   % Gain Matrix
global LastEqnNo            % Variable to keep track of the number of equation incorporated
global KKord
global last_del_g

g0 = g(this_KK);

for a=1:stateVarNo,
    tmp_KK = this_KK;
    d_kk = (1/min(abs(last_del_g(a,:))))*1e-3;
    %if tmp_KK(a) == 0,
        %d_kk = 1e-3;
        %else
        %d_kk = abs(tmp_KK(a))*1e-3;
        %end
    d_kk = 1e-3;
    tmp_KK(a) = tmp_KK(a) + d_kk;
    tmp_g = g(tmp_KK);
    for b=1:stateVarNo,
        del_g(a,b) = (tmp_g(b) - g0(b))/d_kk;
    end
end

last_del_g = del_g;

% ----------------------------
%       End of Function
% ============================


function isThisKKokay = isThisKKokay(this_KK)

%
% given a particular K and lamb, this function calculates the vector g
%
%
%               --                                              --
%              ||                        |                       ||
%              || inv(A-C2*K)*(B+C1*K)   |     inv(A-C2*K)*C0*K  ||
%              ||                        |                       ||
%    g =   eig ||------------------------|-----------------------||
%              ||                        |                       ||
%              ||             I          |         0             ||
%              ||                        |                       ||
%               --                                              --
%


% State space formulation
global stateVarNo           % No of state variables
global ctrlParmNo           % No of control parameters
global ExpressionVectorSize % No of elements in an 'expression vector'
global ss                   % State vector
global AA BB CC0 CC1 CC2 DD % System equation matrices
global KK                   % Gain Matrix
global LastEqnNo            % Variable to keep track of the number of equation incorporated
global lamb


%for n = 1:stateVarNo,
    tmpMat = inv(AA-CC2*this_KK);
    MM11 = tmpMat*(BB+CC1*this_KK);
    MM12 = tmpMat*CC0*this_KK;
    MM21 = eye(stateVarNo);
    MM22 = zeros(stateVarNo);
    MM = [ MM11 MM12 ; MM21 MM22 ];    % / (0.5e100/stateVarNo);
    %MMord = max(max(abs(MM)));
    %MM = MM/lamb(n);
    %rank(MM),
    eigenVals = eig(MM);
    %pause;
    
    isThisKKokay = 1;
    for n = 1:2*stateVarNo,
        if real(eigenVals(n)) > 0
            isThisKKokay = 0;
        end
    end

% ----------------------------
%       End of Function
% ============================
