function PolePlaceIga(lambda)
%
% Places the poles for Integral feedback control on the left of imaginary axis using genetic algorithm
% Fitness function calculated using 'lambda'
%
% Integral control =>
%           uu = KK * Integration[ss * dt]
%
% Can be used when C2 is not zero.
%
% --------------------------------------------------------------------
% Given the position of the poles, 'lamb', in the cplx plane, determines the gain matrix K
% K = [k1 k2 k3 ... kn]
%
% The equation to be satisfied is,
%
%         --                                                             --
%        ||                                |                              ||
%        || inv(A-C2*K)*(B+C1*K) - lamb*I  |     inv(A-C2*K)*C0*K         ||
%        ||                                |                              ||
%    det ||--------------------------------|------------------------------||  =  0
%        ||                                |                              ||
%        ||             I                  |           -lamb*I            ||
%        ||                                |                              ||
%         --                                                             --
%
% There are 2n poles (i.e. 2n 'lamb's), which gives 2n equations.
% But if we assume that A, B, C0, C1, C2 and K have only real elements,
% 'lamb' will consist of complex conjugates.
% Hence, lamb has n elements.
% Solve for K using Newton Raphson method of iteration
% where n = stateVarNo

% NOTE: all values of 'lambda' should fall in -ve real, +ve complex quadrant.

% State space formulation
global stateVarNo           % No of state variables
global ctrlParmNo           % No of control parameters
global ExpressionVectorSize % No of elements in an 'expression vector'
global ss                   % State vector
global AA BB CC0 CC1 CC2 DD % System equation matrices
global KK                   % Gain Matrix
global LastEqnNo            % Variable to keep track of the number of equation incorporated
global lamb KKord

[tmp, indx] = sort(real(lambda));
lamb = lambda(indx);

PopulationSize = 1000;
SurvivalProportion = 0.4;
CrossSteps = 3;
MutationPeriod = 50;
MutationIntensity = 0.2;
maxGeneration = 100;

%%KKord = min([max(max(abs(BB)))/max(abs(CC0)) max(max(abs(BB)))/max(abs(CC1)) max(max(abs(AA)))/max(abs(CC2))]);
KKord = 100;
for a=1:PopulationSize,
    tmp_KK(a,:) = (ones(1, stateVarNo) - rand(1, stateVarNo)/2) * KKord;  % intial population of K
end
tmp_KK(1,:) = zeros(1, stateVarNo); % One member representing no gain

isOkayFlag = 0;
OkayIndex = -1;
for a=1:maxGeneration,
    
    % Evaluating fitness
    for b=1:PopulationSize,
        [KKfitness(b), isKKOkay(b)]=fitness(tmp_KK(b,:));
        if isKKOkay(b)==1
            isOkayFlag = 1;
            OkayIndex = b;
            break;
        end
    end
    
    if isOkayFlag==1
        break;
    end
    
    % Sorting by fitness
    [tmp, indx] = sort(KKfitness);
    tmp_KK = tmp_KK(indx,:);
    
    % Population that will survive
    OldPopulationIndex = round(SurvivalProportion * PopulationSize);
    
    % Mutation
    if mod(a+1,MutationPeriod)==0
        mutationIndex = 1 + round(rand*(PopulationSize-1));
        for b=1: 1 + round((stateVarNo-1)*MutationIntensity),
            tmp_KK(NewPopulationIndex, 1 + round(rand*(stateVarNo-1))) = (1 - rand/2)*KKord;
        end
    end

    % Reproducing the higher fitness individuals and killing the lower fitness ones
    for NewPopulationIndex = OldPopulationIndex+1:PopulationSize,
        % Choosing parents
        Parent2Index = 1 + round(rand*(OldPopulationIndex-1));
        Parent1Index = Parent2Index + round(rand*(OldPopulationIndex-Parent2Index));
        % Crossing
        for b=1:stateVarNo,
            gene1proportion = round(rand*(CrossSteps-1));
            gene2proportion = 1 - gene1proportion;
            tmp_KK(NewPopulationIndex,b) = gene1proportion*tmp_KK(Parent1Index,b) + gene2proportion*tmp_KK(Parent2Index,b);
        end
    end
    
    
    if mod(a+1,10)==0
        sprintf('No solution till Generation number %d', a)
    end
end

if a==maxGeneration
    sprintf('WARNING: No solution for KK found!!!')
else
    sprintf('*** Solution in Generation number %d', a)
    KK = tmp_KK(OkayIndex,:);
end

% ----------------------------
%       End of Function
% ============================




function [fitnessOfThisKK, isThisKKokay]=fitness(this_KK)

%
% given a particular K and lamb, this function calculates the vector g
%
%
%               --                                              --
%              ||                        |                       ||
%              || inv(A-C2*K)*(B+C1*K)   |     inv(A-C2*K)*C0*K  ||
%              ||                        |                       ||
%    g =   eig ||------------------------|-----------------------||
%              ||                        |                       ||
%              ||             I          |         0             ||
%              ||                        |                       ||
%               --                                              --
%


% State space formulation
global stateVarNo           % No of state variables
global ctrlParmNo           % No of control parameters
global ExpressionVectorSize % No of elements in an 'expression vector'
global ss                   % State vector
global AA BB CC0 CC1 CC2 DD % System equation matrices
global KK                   % Gain Matrix
global LastEqnNo            % Variable to keep track of the number of equation incorporated
global lamb


%for n = 1:stateVarNo,
    tmpMat = inv(AA-CC2*this_KK);
    MM11 = tmpMat*(BB+CC1*this_KK);
    MM12 = tmpMat*CC0*this_KK;
    MM21 = eye(stateVarNo);
    MM22 = zeros(stateVarNo);
    MM = [ MM11 MM12 ; MM21 MM22 ];    % / (0.5e100/stateVarNo);
    %MMord = max(max(abs(MM)));
    %MM = MM/lamb(n);
    %rank(MM),
    eigenVals = eig(MM);
    %pause;
    
    fitnessOfThisKK = 0;
    isThisKKokay = 1;
    for n = 1:2*stateVarNo,
        fitnessOfThisKK = fitnessOfThisKK + real(eigenVals(n))*abs(real(eigenVals(n)));
        if real(eigenVals(n)) > 0
            isThisKKokay = 0;
            fitnessOfThisKK = fitnessOfThisKK + max(abs(real(eigenVals)))*10;
        end
    end
    fitnessOfThisKK = -fitnessOfThisKK / (2*stateVarNo);

% ----------------------------
%       End of Function
% ============================

