function PolePlaceIgame

%
% Places the poles for Integral feedback control on the left of imaginary axis using genetic algorithm
%
% Integral control =>
%           uu = KK * Integration[ss * dt]
%
% Can be used when C2 is not zero.
%
% --------------------------------------------------------------------
% Given the position of the poles, 'lamb', in the cplx plane, determines the gain matrix K
% K = [k1 k2 k3 ... kn]
%
% The equation to be satisfied is,
%
%         --                                                             --
%        ||                                |                              ||
%        || inv(A-C2*K)*(B+C1*K) - lamb*I  |     inv(A-C2*K)*C0*K         ||
%        ||                                |                              ||
%    det ||--------------------------------|------------------------------||  =  0
%        ||                                |                              ||
%        ||             I                  |           -lamb*I            ||
%        ||                                |                              ||
%         --                                                             --
%
% There are 2n poles (i.e. 2n 'lamb's), which gives 2n equations.
% But if we assume that A, B, C0, C1, C2 and K have only real elements,
% 'lamb' will consist of complex conjugates.
% Hence, lamb has n elements.
% where n = stateVarNo

% State space formulation
global stateVarNo           % No of state variables
global ctrlParmNo           % No of control parameters
global ExpressionVectorSize % No of elements in an 'expression vector'
global ss                   % State vector
global AA BB CC0 CC1 CC2 DD % System equation matrices
global KK                   % Gain Matrix

tmp_KK = zeros(1, stateVarNo);

maxIter=1000;
for a=1:maxIter,
    isThisKKok = isThisKKokay(tmp_KK);
    if isThisKKok==1
        break;
    elseif isThisKKok==-1
        sprintf('WARNING!! Divergence in iteration no.: %d -- Scaling K by 1e-10!!', a)
        tmp_KK = tmp_KK * 1e-10;
    end

    % Finding best scores and corrosponding changes of the present set of K
    KK_score = zeros(1, stateVarNo);
    delta_KK = zeros(1, stateVarNo);
    for b=1:stateVarNo,
        [delta, score] = findBestDelta3(tmp_KK, b);
        KK_score(b) = score;
        delta_KK(b) = delta;
        sprintf('turn number=%d , K index=%d : score=%d , delta=%d', a, b, score, delta)
    end
        
    [max_score, winner_index] = max(KK_score);
    tmp_KK(winner_index) = tmp_KK(winner_index) + delta_KK(winner_index);
    
    sprintf('No solution till turn number %d :  Last winner score = %d', a, max_score)
end


if a==maxIter
    sprintf('WARNING: No solution for KK found!!!')
else
    sprintf('*** Solution in turn number %d', a)
    KK = tmp_KK;
end


% ----------------------------
%       End of Function
% ============================



function [g]=g(this_KK)

%
% given a particular K and lamb, this function calculates the vector g
%
%
%               --                                              --
%              ||                        |                       ||
%              || inv(A-C2*K)*(B+C1*K)   |     inv(A-C2*K)*C0*K  ||
%              ||                        |                       ||
%    g =   eig ||------------------------|-----------------------||
%              ||                        |                       ||
%              ||             I          |         0             ||
%              ||                        |                       ||
%               --                                              --
%


% State space formulation
global stateVarNo           % No of state variables
global ctrlParmNo           % No of control parameters
global ExpressionVectorSize % No of elements in an 'expression vector'
global ss                   % State vector
global AA BB CC0 CC1 CC2 DD % System equation matrices
global KK                   % Gain Matrix
global LastEqnNo            % Variable to keep track of the number of equation incorporated
global lamb


%for n = 1:stateVarNo,
    tmpMat = inv(AA-CC2*this_KK);
    MM11 = tmpMat*(BB+CC1*this_KK);
    MM12 = tmpMat*CC0*this_KK;
    MM21 = eye(stateVarNo);
    MM22 = zeros(stateVarNo);
    MM = [ MM11 MM12 ; MM21 MM22 ];    % / (0.5e100/stateVarNo);
    eigenVals = eig(MM);
    g = sort(real(eigenVals));



% ----------------------------
%       End of Function
% ============================


function  [delta, score] = findBestDelta3(this_KK, Kindex)
% search for highest score by multiple partition search

no_of_points = 1000;
delta_max = 1e25;
delta_min = -1e25;
delta_gap = (delta_max-delta_min)/(no_of_points+1);

delta_points = delta_min:delta_gap:delta_max;

for a=1:6,
    for b=1:no_of_points,
        score_points(b) = getScore(this_KK, Kindex, delta_points(b));
    end
    [max_score, max_index] = max(score_points);
    the_best_delta = delta_points(max_index);
    delta_max = the_best_delta+delta_gap;
    delta_min = the_best_delta-delta_gap;
    delta_gap = 2*delta_gap/(no_of_points+1);
    delta_points = delta_min:delta_gap:delta_max;
end

delta = the_best_delta;
score = getScore(this_KK, Kindex, delta);

% ----------------------------
%       End of Function
% ============================


function [score] = getScore(this_KK, Kindex, delta)
global stateVarNo

% Parameters in calculating the score
gain_in_positive = 1e-3;
gain_in_negetive = 0.5*1e-3;
gain_in_cross = 1;
% -----------------------------------

tempKK = this_KK;
old_g = g(tempKK);
tempKK(Kindex) = tempKK(Kindex) + delta;
new_g = g(tempKK);
delta_g = old_g - new_g;
gain_in_cross = gain_in_cross*max(abs(delta_g));

score=0;
for a=1:2*stateVarNo,
    old_sign = sign(old_g(a));
    new_sign = sign(new_g(a));
    if old_sign ~= new_sign
        if old_sign == 1
            score = score + gain_in_cross;
        elseif new_sign == 1
            score = score - gain_in_cross;
        end
    end
    if new_sign == 1
        score = score + gain_in_positive*delta_g(a);
    else
        score = score + gain_in_negetive*delta_g(a);
    end
end

score = score / gain_in_cross;


% ----------------------------
%       End of Function
% ============================




function isThisKKokay = isThisKKokay(this_KK)

%
% given a particular K and lamb, this function calculates the vector g
%
%
%               --                                              --
%              ||                        |                       ||
%              || inv(A-C2*K)*(B+C1*K)   |     inv(A-C2*K)*C0*K  ||
%              ||                        |                       ||
%    g =   eig ||------------------------|-----------------------||
%              ||                        |                       ||
%              ||             I          |         0             ||
%              ||                        |                       ||
%               --                                              --
%


% State space formulation
global stateVarNo           % No of state variables
global ctrlParmNo           % No of control parameters
global ExpressionVectorSize % No of elements in an 'expression vector'
global ss                   % State vector
global AA BB CC0 CC1 CC2 DD % System equation matrices
global KK                   % Gain Matrix
global LastEqnNo            % Variable to keep track of the number of equation incorporated
global lamb


%for n = 1:stateVarNo,
    tmpMat = inv(AA-CC2*this_KK);
    MM11 = tmpMat*(BB+CC1*this_KK);
    MM12 = tmpMat*CC0*this_KK;
    MM21 = eye(stateVarNo);
    MM22 = zeros(stateVarNo);
    MM = [ MM11 MM12 ; MM21 MM22 ];    % / (0.5e100/stateVarNo);
    try
        eigenVals = eig(MM);
    catch
        isThisKKokay = -1;
        return;
    end
    
    isThisKKokay = 1;
    for n = 1:2*stateVarNo,
        if real(eigenVals(n)) > 0
            isThisKKokay = 0;
        end
    end

% ----------------------------
%       End of Function
% ============================
