function PolePlaceP(lambda)
%
% Places the poles for proportional feedback control
% i.e. when,
%           uu = KK * ss
%
% Can be used when C2 is not zero.
% WARNING: proportional feedback control should be avoided!!
%
% --------------------------------------------------------------------
% Given the position of the poles, 'lamb', in the cplx plane, determines the gain matrix K
% K = [k1 k2 k3 ... kn]
%
% The equation to be solved for K is,
%
%         --                                                                 --
%        ||                                 |                                 ||
%        || inv(C2*K)*(A-C1*K) - lamb*I     |     -inv(C2*K)*(B+C0*K)         ||
%        ||                                 |                                 ||
%    det ||---------------------------------|---------------------------------||  =  0
%        ||                                 |                                 ||
%        ||             I                   |           -lamb*I               ||
%        ||                                 |                                 ||
%         --                                                                 --
%
% There are 2n poles (i.e. 2n 'lamb's), which gives 2n equations.
% But if we assume that A, B, C0, C1, C2 and K have only real elements,
% 'lamb' will consist of complex conjugates.
% Hence, lamb has n elements.
% Solve for K using Newton Raphson method of iteration
% where n = stateVarNo

% State space formulation
global stateVarNo           % No of state variables
global ctrlParmNo           % No of control parameters
global ExpressionVectorSize % No of elements in an 'expression vector'
global ss                   % State vector
global AA BB CC0 CC1 CC2 DD % System equation matrices
global KK                   % Gain Matrix
global LastEqnNo            % Variable to keep track of the number of equation incorporated
global lamb

lamb = lambda;

tmp_KK = ones(1, stateVarNo) - rand(1, stateVarNo)/2;  % intial guess of K

error_chk = .00001;

the_last_error = inf;
for a=1:1000,

    last_tmp_KK = tmp_KK;
    
    Mat_del_g = del_g(tmp_KK);
    tmp_KK = tmp_KK - g(tmp_KK)*inv(Mat_del_g);
    
    if (dot(tmp_KK, tmp_KK) == 0) && (dot(last_tmp_KK, last_tmp_KK) == 0)
        break;
    end
    
    the_error = dot((tmp_KK-last_tmp_KK), (tmp_KK-last_tmp_KK))/dot((tmp_KK+last_tmp_KK), (tmp_KK+last_tmp_KK));
    if the_error < error_chk
        break;
    end
    
    if mod(a+1,10)==0
        sprintf('Iteration no.: %d -- Error = %d', a, the_error)
    end
    if isnan(the_error) || isinf(the_error)  % divergence
        sprintf('WARNING!! Divergence in iteration no.: %d -- Re-initiating iteration!!', a)
        %tmp_ddq_p = ddq_p + max(abs(ddq_p))*(ones(1, modesNo) - rand(1, modesNo)/2);
        tmp_KK = ones(1, stateVarNo) - rand(1, stateVarNo)/2;
        a = 0;
    end
    the_last_error = the_error;
end
sprintf('Convergence in %d iterations', a)

KK = tmp_KK;


% ----------------------------
%       End of Function
% ============================




function [g]=g(this_KK)
global lamb

%
% given a particular K and lamb, this function calculates the vector g
%
% 
%                --                                                                 --
%               ||                                 |                                 ||
%               || inv(C2*K)*(A-C1*K) - lamb(i)*I  |     -inv(C2*K)*(B+C0*K)         ||
%               ||                                 |                                 ||
%  g(i) =   det ||---------------------------------|---------------------------------||  =  0
%               ||                                 |                                 ||
%               ||             I                   |           -lamb(i)*I            ||
%               ||                                 |                                 ||
%                --                                                                 --
%
% i = 1 to stateVarNo
%

% State space formulation
global stateVarNo           % No of state variables
global ctrlParmNo           % No of control parameters
global ExpressionVectorSize % No of elements in an 'expression vector'
global ss                   % State vector
global AA BB CC0 CC1 CC2 DD % System equation matrices
global KK                   % Gain Matrix
global LastEqnNo            % Variable to keep track of the number of equation incorporated


for n = 1:stateVarNo,
    tmpMat = inv(CC2*this_KK);
    MM11 = tmpMat*(AA-CC1*this_KK) - lamb(n)*eye(stateVarNo);
    MM12 = -tmpMat*(BB+CC0*this_KK);
    MM21 = eye(stateVarNo);
    MM22 = - lamb(n)*eye(stateVarNo);
    MM = [ MM11 MM12 ; MM21 MM22 ];
    tmpg(n) = real(det(MM));
end

g = tmpg;

% ----------------------------
%       End of Function
% ============================



function [del_g]=del_g(this_KK)

% returns the matrix
%
% del * g' =
%           d(g(1))/d(KK(1))           d(g(2))/d(KK(1))           ...      d(g(stateVarNo))/d(KK(1))
%           d(g(1))/d(KK(2))           d(g(2))/d(KK(2))           ...      d(g(stateVarNo))/d(KK(2))
%                  :                           :                                       :
%                  :                           :         d(g(b))/d(KK(a))              :
%                  :                           :                                       :
%           d(g(1))/d(KK(stateVarNo))  d(g(2))/d(KK(stateVarNo))   ...      d(g(modesNo))/d(KK(stateVarNo))

% State space formulation
global stateVarNo           % No of state variables
global ctrlParmNo           % No of control parameters
global ExpressionVectorSize % No of elements in an 'expression vector'
global ss                   % State vector
global AA BB CC0 CC1 CC2 DD % System equation matrices
global KK                   % Gain Matrix
global LastEqnNo            % Variable to keep track of the number of equation incorporated

g0 = g(this_KK);

for a=1:stateVarNo,
    tmp_KK = this_KK;
    if tmp_KK(a) == 0,
        d_kk = 1e-10;
    else
        d_kk = abs(tmp_KK(a))*1e-6;
    end
    tmp_KK(a) = tmp_KK(a) + d_kk;
    tmp_g = g(tmp_KK);
    for b=1:stateVarNo,
        del_g(a,b) = (tmp_g(b) - g0(b))/d_kk;
    end
end

% ----------------------------
%       End of Function
% ============================



