function generateEquation(eqnVector)
% adds an equation to the global state-space matrices
% 
% the equation is,
%           dot(eqnVector, VV) = 0
% where,
% VV = [ independent_term
%        DDcp(1) DDcp(2) ... DDcp(ctrlParmNo)    Dcp(1) Dcp(2) ... Dcp(ctrlParmNo)   cp(1) cp(2) ... cp(ctrlParmNo)
%        Du Dv Dw    u v w    DDpAng DDqAng DDrAng    DpAng DqAng DrAng    pAng qAng rAng
%        DDq_p(1) DDq_p(2) ... DDq_p(modesNo)   Dq_p(1)  Dq_p(2) ...  Dq_p(modesNo)    q_p(1)  q_p(2) ...  q_p(modesNo) ]
%
% The global system equation is,
%           AA * Dss =  BB * ss  +  CC0 * uu  +  CC1 * Duu  +  CC2 * DDuu  +  DD
%
%  AA, BB -> stateVarNo x stateVarNo
%  CC0, CC1, CC2 -> stateVarNo x ctrlParmNo
%  DD, s -> stateVarNo x 1
%  u  -> ctrlParmNo x 1
%
% where ss is the state vector given by,
%           ss = [ u v w   DpAng DqAng DrAng   pAng qAng rAng   Dq_p(1) Dq_p(2) ... Dq_p(modesNo)   q_p(1) q_p(2) ... q_p(modesNo)]'
%
% and uu is the input vector of control parameters given by,
%           uu = [ cp(1)  cp(2)  ...  cp(ctrlParmNo) ]'
%
% Note: D = d/dt
%

global modesNo
% State space formulation
global stateVarNo           % No of state variables
global ctrlParmNo           % No of control parameters
global ExpressionVectorSize % No of elements in an 'expression vector'
global ss                   % State vector
global AA BB CC0 CC1 CC2 DD % System equation matrices
global LastEqnNo            % Variable to keep track of the number of equation incorporated

LastEqnNo = LastEqnNo + 1;

% u v w
for a = 1:3,
    AA(LastEqnNo, a) = eqnVector(1+3*ctrlParmNo+a);
    BB(LastEqnNo, a) = -eqnVector(4+3*ctrlParmNo+a);
end

% pAng qAng rAng
for a = 1:3,
    AA(LastEqnNo, 3+a) = eqnVector(7+3*ctrlParmNo+a);
end
for a = 1:6,
    BB(LastEqnNo, 3+a) = -eqnVector(10+3*ctrlParmNo+a);
end

% q_p(i), i = 1 to modesNo
for a = 1:modesNo,
    AA(LastEqnNo, 9+a) = eqnVector(16+3*ctrlParmNo+a);
end
for a = 1:2*modesNo,
    BB(LastEqnNo, 9+a) = -eqnVector(16+3*ctrlParmNo+modesNo+a);
end

% Control parameters
for a = 1:ctrlParmNo,
    CC0(LastEqnNo) = -eqnVector(1+2*ctrlParmNo+a);
    CC1(LastEqnNo) = -eqnVector(1+ctrlParmNo+a);
    CC2(LastEqnNo) = -eqnVector(1+a);
end
% const term
DD(LastEqnNo) = -eqnVector(1);


% Normalizing
AAfactor = max(abs(AA(LastEqnNo,:)));
AA(LastEqnNo,:) = AA(LastEqnNo,:) / AAfactor;
BB(LastEqnNo,:) = BB(LastEqnNo,:) / AAfactor;
CC0(LastEqnNo,:) = CC0(LastEqnNo,:) / AAfactor;
CC1(LastEqnNo,:) = CC1(LastEqnNo,:) / AAfactor;
CC2(LastEqnNo,:) = CC2(LastEqnNo,:) / AAfactor;
DD(LastEqnNo,:) = DD(LastEqnNo,:) / AAfactor;
