function globals
% ==================================================================
% Global variable declarations
% ==================================================================

% Control parameters
global del_p d_del_p dd_del_p   % Thrust angle in pitch plane
global del_y d_del_y dd_del_p   % Thrust angle in yaw plane
global Tc Ts                    % Thrust values

% Structural Constants
global materialE materialRho airRho
global m0 L0 csA meu
global xcg ycg zcg
global Ixx Iyy Izz Ixy Iyz Izx
global LC                     % Distance of the coordinate origin from hind end of the missile
global LR                     % Dist between engine cg and the hinge to which the engine is attached
global La                     % Distance between origine and nose tip of vehicle
global mR                     % Mass of engine
global mM                     % Mass of missile without engine
global I0 IR                  % moment of inertia of engine abt cg and hinge respectively

% Net external Forces and Moments --> Time dependant scalers
global Fx Fy Fz
global Mx My Mz

% Angle, Velocity and accleration components --> constants
global U0 V0 W0
global P0 Q0 R0
global dU0 dV0 dW0
global dP0 dQ0 dR0
global pAng0 qAng0 rAng0
% Purturbation Angle, Velocity and accleration components --> Time dependant scalers
global pAng qAng rAng
global u v w
global p q r
global du dv dw
global dp dq dr

% Structural Variables
global modesNo          % No. of modes considered on each of pitch plane and yaw plane
global beta Amode Bmode % Mode shape definitions
% Pitch plane parameters
global zeta_p       % Damping coefficient
global omgMode_p    % Modal frequency for pitch plane modes
global q_p          % Coefficients of Mode shapes - 'modesNo' Time dependant scalers
global dq_p         % Time derivatives of q_p
global ddq_p        % 2nd time derivatives of q_p
global F_p          % Force per unit length in Pitch plane = force along Z axis --> Function of length
global M_p          % Moment per unit length in Pitch plane = moment along Y axis --> Function of length
global F_p_ddq_coef F_p_dq_coef F_p_q_coef % Global coeficients
global DC_p DC_p_ddq_coef DC_p_dq_coef DC_p_q_coef  % Derivative of moments due to couple along length
% Yaw plane parameters
global zeta_y       % Damping coefficient
global omgMode_y    % Modal frequency for yaw plane modes
global q_y          % Coefficients of Mode shapes - 'modesNo' Time dependant scalers
global dq_y         % Time derivatives of q_y
global ddq_y        % 2nd time derivatives of q_y
global F_y          % Force per unit length in Yaw plane = force along Y axis --> Function of length
global M_y          % Moment per unit length in Yaw plane = moment along Z axis --> Function of length
%
global F_a          % Force per unit length in axial direction = force along X axis --> Function of length
global M_a          % Moment per unit length due to twisting couples = moment along X axis --> Function of length

% Discreetization constants
global dt
global dl lSteps
global d_ddq_p

% Runtime variables
global time
global Tdata1 Tdata2

% State space formulation
global stateVarNo           % No of state variables
global ctrlParmNo           % No of control parameters
global ExpressionVectorSize % No of elements in an 'expression vector'
global ss rr                % State vector
global AA BB CC0 CC1 CC2 DD % System equation matrices
global KK uu Duu DDuu last_Duu      % Gain Matrix & Inputs for Integral Control
global MM_full NN_full      % For the full integral system
global LastEqnNo            % Variable to keep track of the number of equation incorporated
global last_Duu             % Required for Integral control

% System and control types
global isTimeInvariant      % 1 => time invariant system --> reduces calculations considerably
global isIntegralControl    % 1 => uses Integral control --> for systems involving dd_del_p
global doFull               % 1 => solve the full integral system at a single time

% ==================================================================
% Global Variables values assignment
% ==================================================================

airRho = 1.2;
materialE = 7e10;               % Young's modulus
materialRho = 2600;             % Density
%materialE = 2e11;               % Young's modulus
%materialRho = 7800;             % Density
L0 = 1;                         % Length
csA = pi*0.1*0.1/4;             % C.S. area
mR = .1;                        % Engine mass
m0 = materialRho*L0*csA + mR;   % total mass
mM = m0 - mR;
meu = mM/L0;

Ixx = 2*pi*0.1*0.1*0.1*0.1*mM/csA;
Iyy = L0*L0*mM/12;
Izz = Iyy;
Ixy = 0;
Iyz = 0;
Izx = 0;
xcg = 0;    % Setting origin at the cg
ycg = 0;
zcg = 0;

LC = L0/2;  % Distance of the coordinate origin from hind end of the missile
LR = 0.04;  % Dist between engine cg and the hinge to which the engine is attached
La = L0 - LC;  % Distance between origine and nose tip of vehicle

I0 = mR*LC*LC; %100;   % engine moment of inertia about origine
IR = I0 + mR*LR*LR;

%%I_p = 0.5*(Ixx + Iyy - Izz);
I_p = pi*0.1*0.1*0.1*0.1/16;
I_y = I_p; %%0.5*(Ixx - Iyy + Izz);

modesNo = 10;
for n = 1:modesNo,
    zeta_p(n) = 0.01;
end

ctrlParmNo = 1;              % del_p is presently the only control parameter
stateVarNo = 9 + 2*modesNo;  % No of state variables % excluding sloshing
ExpressionVectorSize = 1 + 3*ctrlParmNo + 15 + 3*modesNo;

dt = 0.0000001; %1e-28;        % will depend max value of omega:  dt << 2*pi/omg_max
dl = 0.01;
lSteps = L0 / dl;
d_ddq_p = 0.000001;

% ==================================================================
% Global Variables values computation
% ==================================================================

% Finding betaL: Roots of cos(betaL)*cosh(betaL)=1
% 0th Root: 0
% 1nd root between 0.5*2pi and 1*2pi
% 2rd root between 1*2pi and 1.5*2pi
% 3th root between 1.5*2pi and 2*2pi
% ...
% nth root between (n)pi and (n+1)pi

% Calculating for 1st, 2nd and 3rd modes
for n = 1:3,
    lowBetaL = n*pi;
    hiBetaL = (n+1)*pi;
    lowVal = cos(lowBetaL)*cosh(lowBetaL) - 1;
    hiVal = cos(hiBetaL)*cosh(hiBetaL) - 1;
    lowSign = sign(lowVal);
    hiSign = sign(hiVal);
    while true,
        searchBetaL = (lowBetaL*hiVal - hiBetaL*lowVal)/(hiVal-lowVal);
        searchVal = cos(searchBetaL)*cosh(searchBetaL) - 1;
        if abs(searchVal) < 1e-10
            break;
        end
        searchSign = sign(searchVal);
        if searchSign == lowSign
            lowBetaL = searchBetaL;
            lowVal = searchVal;
        else
            lowBetaL = searchBetaL;
            hiVal = searchVal;
        end
    end
    beta(n) = searchBetaL/L0;
    Amode(n) = -(sinh(beta(n)*L0)-sin(beta(n)*L0));
    Bmode(n) = (cosh(beta(n)*L0)-cos(beta(n)*L0));
    omgMode_p(n) = beta(n)*beta(n)*sqrt((materialE*I_p)/(materialRho*csA));
    sprintf('Mode %d: beta=%1.10f , A=%d , B=%d, omg=%d', n, beta(n), Amode(n), Bmode(n), omgMode_p(n)),
end
% for n th mode with n>3, betaL = (n + 0.5)pi (approx.)
for n = 4:modesNo,
    beta(n) = (n + 0.5)*pi/L0;
    Amode(n) = -(sinh(beta(n)*L0)-sin(beta(n)*L0));
    Bmode(n) = (cosh(beta(n)*L0)-cos(beta(n)*L0));
    omgMode_p(n) = beta(n)*beta(n)*sqrt((materialE*I_p)/(materialRho*csA));
    sprintf('Mode %d: beta=%1.10f , A=%d , B=%d, omg=%d', n, beta(n), Amode(n), Bmode(n), omgMode_p(n)),
end
